@@ -1,4 +1,6 @@
 //! Archive compression algorithms
 
+pub mod rar;
+pub mod sevenz;
 pub mod tar;
 pub mod zip;
