@@ -10,11 +10,10 @@ use std::{
     thread,
 };
 
-use filetime::{set_file_mtime, FileTime};
+use filetime_creation::{set_file_mtime, FileTime};
 use fs_err as fs;
 use same_file::Handle;
 use time::OffsetDateTime;
-use ubyte::ToByteUnit;
 use zip::{self, read::ZipFile, DateTime, ZipArchive};
 
 use crate::{
@@ -22,7 +21,7 @@ use crate::{
     info,
     list::FileInArchive,
     utils::{
-        self, cd_into_same_dir_as, get_invalid_utf8_paths, pretty_format_list_of_paths, strip_cur_dir,
+        self, cd_into_same_dir_as, get_invalid_utf8_paths, pretty_format_list_of_paths, strip_cur_dir, Bytes,
         EscapedPathDisplay, FileVisibilityPolicy,
     },
     warning,
@@ -74,7 +73,7 @@ where
                         inaccessible,
                         "{:?} extracted. ({})",
                         file_path.display(),
-                        file.size().bytes()
+                        Bytes::new(file.size()),
                     );
                 }
 
@@ -168,7 +167,8 @@ where
     for filename in input_filenames {
         let previous_location = cd_into_same_dir_as(filename)?;
 
-        // Safe unwrap, input shall be treated before
+        // Unwrap safety:
+        //   paths should be canonicalized by now, and the root directory rejected.
         let filename = filename.file_name().unwrap();
 
         for entry in file_visibility_policy.build_walker(filename) {
@@ -176,7 +176,7 @@ where
             let path = entry.path();
 
             // If the output_path is the same as the input file, warn the user and skip the input (in order to avoid compression recursion)
-            if let Ok(ref handle) = output_handle {
+            if let Ok(handle) = &output_handle {
                 if matches!(Handle::from_path(path), Ok(x) if &x == handle) {
                     warning!(
                         "The output file and the input file are the same: `{}`, skipping...",
@@ -209,8 +209,13 @@ where
             #[cfg(unix)]
             let options = options.unix_permissions(metadata.permissions().mode());
 
+            let entry_name = path.to_str().ok_or_else(|| {
+                FinalError::with_title("Zip requires that all directories names are valid UTF-8")
+                    .detail(format!("File at '{path:?}' has a non-UTF-8 name"))
+            })?;
+
             if metadata.is_dir() {
-                writer.add_directory(path.to_str().unwrap().to_owned(), options)?;
+                writer.add_directory(entry_name, options)?;
             } else {
                 #[cfg(not(unix))]
                 let options = if is_executable::is_executable(path) {
@@ -220,10 +225,11 @@ where
                 };
 
                 let mut file = fs::File::open(path)?;
-                writer.start_file(
-                    path.to_str().unwrap(),
-                    options.last_modified_time(get_last_modified_time(&file)),
-                )?;
+
+                // Updated last modified time
+                let last_modified_time = options.last_modified_time(get_last_modified_time(&file));
+
+                writer.start_file(entry_name, last_modified_time)?;
                 io::copy(&mut file, &mut writer)?;
             }
         }
