@@ -1,7 +1,10 @@
 #[macro_use]
 mod utils;
 
-use std::{iter::once, path::PathBuf};
+use std::{
+    iter::once,
+    path::{Path, PathBuf},
+};
 
 use fs_err as fs;
 use parse_display::Display;
@@ -16,6 +19,8 @@ use crate::utils::{assert_same_directory, write_random_content};
 #[derive(Arbitrary, Debug, Display)]
 #[display(style = "lowercase")]
 enum DirectoryExtension {
+    #[display("7z")]
+    SevenZ,
     Tar,
     Tbz,
     Tbz2,
@@ -65,22 +70,22 @@ fn create_random_files(dir: impl Into<PathBuf>, depth: u8, rng: &mut SmallRng) {
 
     let dir = &dir.into();
 
-    // create 0 to 7 random files
-    for _ in 0..rng.gen_range(0..8u32) {
+    // create 0 to 4 random files
+    for _ in 0..rng.gen_range(0..=4u32) {
         write_random_content(
             &mut tempfile::Builder::new().tempfile_in(dir).unwrap().keep().unwrap().0,
             rng,
         );
     }
 
-    // create more random files in 0 to 3 new directories
-    for _ in 0..rng.gen_range(0..4u32) {
+    // create more random files in 0 to 2 new directories
+    for _ in 0..rng.gen_range(0..=2u32) {
         create_random_files(&tempfile::tempdir_in(dir).unwrap().into_path(), depth - 1, rng);
     }
 }
 
 // compress and decompress a single empty file
-#[proptest(cases = 512)]
+#[proptest(cases = 200)]
 fn single_empty_file(ext: Extension, #[any(size_range(0..8).lift())] exts: Vec<FileExtension>) {
     let dir = tempdir().unwrap();
     let dir = dir.path();
@@ -99,11 +104,15 @@ fn single_empty_file(ext: Extension, #[any(size_range(0..8).lift())] exts: Vec<F
 }
 
 // compress and decompress a single file
-#[proptest(cases = 512)]
+#[proptest(cases = 250)]
 fn single_file(
     ext: Extension,
     #[any(size_range(0..8).lift())] exts: Vec<FileExtension>,
-    #[strategy(proptest::option::of(0i16..12))] level: Option<i16>,
+    #[cfg_attr(not(target_arch = "arm"), strategy(proptest::option::of(0i16..12)))]
+    // Decrease the value of --level flag for `arm` systems, because our GitHub
+    // Actions CI runs QEMU which makes the memory consumption higher.
+    #[cfg_attr(target_arch = "arm", strategy(proptest::option::of(0i16..8)))]
+    level: Option<i16>,
 ) {
     let dir = tempdir().unwrap();
     let dir = dir.path();
@@ -123,10 +132,13 @@ fn single_file(
 }
 
 // compress and decompress a directory with random content generated with create_random_files
-#[proptest(cases = 512)]
+//
+// this one runs only 50 times because there are only `.zip` and `.tar` to be tested, and
+// single-file formats testing is done in the other test
+#[proptest(cases = 50)]
 fn multiple_files(
     ext: DirectoryExtension,
-    #[any(size_range(0..8).lift())] exts: Vec<FileExtension>,
+    #[any(size_range(0..5).lift())] exts: Vec<FileExtension>,
     #[strategy(0u8..4)] depth: u8,
 ) {
     let dir = tempdir().unwrap();
@@ -141,3 +153,26 @@ fn multiple_files(
     ouch!("-A", "d", archive, "-d", after);
     assert_same_directory(before, after, !matches!(ext, DirectoryExtension::Zip));
 }
+
+// test .rar decompression
+fn test_unpack_rar_single(input: &Path) -> Result<(), Box<dyn std::error::Error>> {
+    let dir = tempdir()?;
+    let dirpath = dir.path();
+    let unpacked_path = &dirpath.join("testfile.txt");
+    ouch!("-A", "d", input, "-d", dirpath);
+    let content = fs::read_to_string(unpacked_path)?;
+    assert_eq!(content, "Testing 123\n");
+
+    Ok(())
+}
+
+#[test]
+fn unpack_rar() -> Result<(), Box<dyn std::error::Error>> {
+    let mut datadir = PathBuf::from(std::env::var("CARGO_MANIFEST_DIR")?);
+    datadir.push("tests/data");
+    ["testfile.rar3.rar.gz", "testfile.rar5.rar"]
+        .iter()
+        .try_for_each(|path| test_unpack_rar_single(&datadir.join(path)))?;
+
+    Ok(())
+}
